#!/usr/bin/env perl

use XyceRegression::Tools;

$Tools = XyceRegression::Tools->new();
#$Tools->setDebug(1);
#$Tools->setVerbose(1);

# The input arguments to this script are: 
# $1 = location of Xyce binary
# $2 = location of xyce_verify.pl script
# $3 = location of compare script 
# $4 = location of circuit file to test
# $5 = location of gold standard prn file

# If Xyce does not produce a prn file, then we return exit code 10.
# If Xyce succeeds, but the test fails, then we return exit code 2.
# If the shell script fails for some reason, then we return exit code 1.
# Otherwise we return the exit code of compare or xyce_verify.pl

# Since the shell script runs Xyce and the comparison program, it is
# responsible for capturing any error output from Xyce and the STDOUT & STDERR
# from the comparison program.  The outside script, run_xyce_regression,
# expects the STDERR output from Xyce to go into $CIRFILE.err, the STDOUT
# output from comparison to go into $CIRFILE.prn.out and the STDERR output from
# comparison to go into $CIRFILE.prn.err.  

use Getopt::Long;
use Scalar::Util qw(looks_like_number);

&GetOptions( "verbose!" => \$verbose );
$XYCE=$ARGV[0];
$XYCE_VERIFY=$ARGV[1];
#$XYCE_COMPARE=$ARGV[2];
$CIRFILE=$ARGV[3];

# these are the tolerances used in comparing the measures calculated by this 
# script to those calculated by Xyce.  
my $absTol = 1.0e-8;
my $relTol = 0.001;

# change ending on gold standard file from ".prn" as passed in to ".gs" 
# which is the file storing the measure results.
$GOLDSENS = $CIRFILE . "_dodp.txt.gs";
$DIRECTFILE = $CIRFILE . "0_dodpDirect.txt";

if (defined($verbose)) { $Tools->setVerbose(1); }

sub verbosePrint { $Tools->verbosePrint(@_); }

$retval = -1;
$retval=$Tools->wrapXyce($XYCE,$CIRFILE);
if ($retval != 0) { print "Exit code = $retval\n"; exit $retval; }
if (not -s "$CIRFILE.prn" ) { print "Exit code = 14\n"; exit 14; }

#If this is a VALGRIND run, we don't do our normal verification, we
# merely run "valgrind_check.sh" as if it were xyce_verify.pl

if ($XYCE_VERIFY =~ m/valgrind_check/)
{
    print STDERR "DOING VALGRIND RUN INSTEAD OF REAL RUN!";
    if (system("$XYCE_VERIFY $CIRFILE junk $CIRFILE.prn > $CIRFILE.prn.out 2>&1 $CIRFILE.prn.err"))
    {
        print "Exit code = 2 \n";
        exit 2;
    }
    else
    {
        print "Exit code = 0 \n";
        exit 0;
    }
}

#
# Did we make a direct file
#
if (not -s "$DIRECTFILE" ) { print "Exit code = 17\n"; exit 17; }


# check that the files have the same number of lines
open(DIRRESULTS, "$DIRECTFILE");
open(GOLD_STD, $GOLDSENS);
for ($dirCount=0; <DIRRESULTS>; $dirCount++) { }
for ($gs_Count=0; <GOLD_STD>; $gs_Count++) { }
close(DIRRESULTS);
close(GOLD_STD);

if( ($dirCount != $gs_Count) )
{
  print "Xyce's Direct doesn't match the gold standard\n";
  print "Xyce's Direct line count: $dirCount\n";
  print "Gold Standard line count: $gs_Count\n";
  $retval=2;
}
else
{
  # If the line counts match, then compare in detail.
  # Re-open the direct output file and compare it to the
  # gold standard line by line.
  open(DIRRESULTS, "$DIRECTFILE");
  open(GOLD_STD, $GOLDSENS);

  # Compare data 
  while( ($lineDir=<DIRRESULTS>) && ($line_gs=<GOLD_STD>) )
  {
    # process a line into text and values.
    chop $lineDir;
    # Remove leading spaces on line, otherwise the spaces become 
    # element 0 of "@directXyceData" instead of the first column of data.
    $lineDir =~ s/^\s*//;
    @directXyceData = (split(/[\s,]+/, $lineDir));

    # process a line_gs into text and values.
    chop $line_gs;
    $line_gs =~ s/^\s*//;
    @gsLineOfDataFromXyce = (split(/[\s,]+/, $line_gs));

    if( ($#directXyceData != $#gsLineOfDataFromXyce) )
    {
      print "Xyce's Direct file doesn't match the gold standard\n";
      print "Xyce's Direct output: $lineDir\n";
      print "Gold Standard: $line_gs\n";
      $retval=2;
    }
    else
    {
      # the two files have the same number of items on a line.  
      # compare individual values as scalars  This will have the 
      # effect of 
      for( $i=0; $i<=$#directXyceData; $i++ )
      {
        if (looks_like_number($directXyceData[$i]) && 
            looks_like_number($gsLineOfDataFromXyce[$i]) )
        {
          if( (abs( $directXyceData[$i] - $gsLineOfDataFromXyce[$i] ) < $absTol )) 
          {
             # regular compare
          }
          else
          {
            print "$DIRECTFILE and $GOLDSENS found a numeric difference\n";
            print "Comparing Xyce's direct data \"$directXyceData[$i]\" to GS data \"$gsLineOfDataFromXyce[$i]\" as ";
            print "Failed numeric compare\n";
            $retval=2;
            last;
          }
        }
        elsif( ($directXyceData[$i] eq $gsLineOfDataFromXyce[$i]) )
        {
          # same in string context so ok.
          # print "string compare ok\n";
        }
        else
        {
          print "Elements failed compare:\n";
          print "Xyce Direct produced: \"$directXyceData[$i]\"\n";
          print "Gold standard: \"$gsLineOfDataFromXyce[$i]\"\n";
          $retval=2;
          last;
        }
      
      }
    
    }
  }
  close(DIRRESULTS);
  close(GOLD_STD);
}

if ( $retval != 0 )
{
  verbosePrint "test Failed!\n";
  print "Exit code = $retval\n"; 
  exit $retval;
}
  
verbosePrint "test Passed!\n";
print "Exit code = $retval\n"; 
exit $retval;

